#! /bin/bash
#Mithat Konar - 2009 Jan 15

# Changes dpi references in Xservers files to $1.
# If $2 is passed, $2 is assumed to be a path to an XDM Xservers file.
# If it is not passed, the file "/etc/X11/xdm/Xservers" will be changed.
# Examples:
#   To set system to 96 dpi in "/etc/X11/xdm/Xservers":
#   setDPI.sh 96
#   To set system to 75 dpi in file "./foo/Xservers":
#   setDPI.sh 75 "./foo/Xservers"
# The script will make a backup of the original file.
# But you should still USE WITH GREAT CAUTION.

# exit codes
notEnoughParams=3		# not enough parameters were passed to the script
dpiNotValid=4			# the DPI parameter was not valid (must be non-negative)
outputFileNotValid=5	# the config file does not exist or you do not have write permission
permissionProb=126		# Command invoked cannot execute

# check that user is root, show error if not
effectiveUser=`whoami`
if [ "$effectiveUser" != "root" ] ; then
	echo "$0: Must be root." 1>&2
	exit $permissionProb
fi

# check 1st parameter -- it must be a non-negative number
if [ -z "$1" ]; then
	echo "$0: Not enough parameters" 1>&2
	exit $notEnoughParams
else
	# test that it's a number >= 0
	isNonNegNum=$(echo "$1" | sed "s/[0-9]//g;s/\.//")	# strip out allowed chars
	if [ -n "$isNonNegNum" ] ; then
		echo "$0: DPI must be a non-negative number" 1>&2
		exit $dpiNotValid
	fi
fi

# check 2nd parameter -- default to /etc/X11/xdm/Xservers
if [ -z "$2" ]; then
	cfgFile="/etc/X11/xdm/Xservers"
else
	cfgFile="$2"
fi

# test if file exists and write permission is granted
if [ ! -w "$cfgFile" ]; then
	echo "$0: The file '$cfgFile' does not exist or you do not have write permission." 1>&2
	exit $outputFileNotValid
fi

# set working files
suffix=$(date +%s)
bckpFile="${cfgFile}.${suffix}"
tmpFile=/tmp/set-xdm-DPI.tmp.$$
cp -dp "$cfgFile" "$bckpFile"	# make backup of original file

# First make a temp copy of original file but with any (active) dpi specifiers stripped out:
# In all lines starting with ":[digit]" or "[whitespace]:[digit]", substitute "-dpi [token]" with "".
# The regex below matches in any line beginning with "[whitespace]*:[integer]"
# the phrase "[whitespace]-dpi [start with a digit, grab all following numbers, letters and periods]"
# and substitutes that with "".
sed "/^[ \t]*:[0-9]/ s/[ \t]-dpi [0-9][.a-zA-Z0-9]*//g" "$cfgFile" > "$tmpFile"

# Next, if the first argument is zero, just write the config file and we're done;
# otherwise add the first argument as a DPI option:
# We gotta jump thru some hoops to handle 'non-integer" zero. The below strips away
# 'permitted' characters from $1. If anything is left, the result must be non-zero.
isZero=$(echo "$1" | sed "s/^[+-]//;s/[0]//g;s/\.//")
if [ -z "$isZero" ] ; then
	# copy temp file as is to cfgFile 
	cp -dp "$tmpFile" "$cfgFile"
	# add a comment to the end of the file
	echo -e "# This file was modified by set-xdm-DPI.sh on $(date) to remove the DPI option." >> "$cfgFile"
else
	# append "-dpi $1" to all lines starting with ":[digit]" or "[whitespace]:[digit]
	sed "/^[ \t]*:[0-9]/ s/$/ -dpi $1/g" "$tmpFile" > "$cfgFile"
	# add a comment to the end of the file
	echo -e "# This file was modified by set-xdm-DPI.sh on $(date) to $1 DPI." >> "$cfgFile"
fi

# Finally, remove the tmpFile
rm -f $tmpFile
